from ctypes import *

from pyhdhomerun.utility import ip_int_to_ascii
from pyhdhomerun.constants import *

class TYPE_hdhomerun_discover_device_t(Structure):
    _fields_ = [('ip_addr',     c_uint, 32),
                ('device_type', c_uint, 32),
                ('device_id',   c_uint, 32),
                ('tuner_count', c_ubyte) 
               ]

    @property
    def nice_ip(self):
        return ip_int_to_ascii(self.ip_addr)

    @property
    def nice_device_id(self):
        return ("%X" % self.device_id)

    def __str__(self):
        return ('DISCOVER  ID= [%s]  TYPE= (%d)  IP= [%s]  TUNERS= (%d)' % 
                (self.nice_device_id, self.device_type, self.nice_ip, 
                 self.tuner_count))

class TYPE_hdhomerun_channel_entry_t(Structure):
    def __str__(self):
        return ("CHANNEL-ENTRY  NEXT= [%s]  PREV= [%s]  FREQ= (%d)  CHAN= (%d)"
                "  NAME= [%s]" % ('<present>' if self.next else None, 
                                  '<present>' if self.prev else None, 
                                  self.frequency, self.channel_number, 
                                  ''))#self.name.value))
    
TYPE_hdhomerun_channel_entry_t._fields_ = \
    [('next',           POINTER(TYPE_hdhomerun_channel_entry_t)),
     ('prev',           POINTER(TYPE_hdhomerun_channel_entry_t)),
     ('frequency',      c_uint, 32),
     ('channel_number', c_uint, 16),
     ('name',           c_char * 16) 
    ]

class TYPE_hdhomerun_channel_list_t(Structure):
    _fields_ = [('head', POINTER(TYPE_hdhomerun_channel_entry_t)),
                ('tail', POINTER(TYPE_hdhomerun_channel_entry_t))
               ]

    def __str__(self):
        return ('CHANNEL-LIST  HEAD= [%s]  TAIL= [%s]' % 
                    ('<present>' if self.head else None, 
                     '<present>' if self.tail else None))

#class TYPE_hdhomerun_channelmap_range_t(Structure):
#    _fields_ = [('channel_range_start', c_uint, 16),
#                ('channel_range_end',   c_uint, 16),
#                ('frequency',           c_uint, 32),
#                ('spacing',             c_uint, 32),
#               ]

#class TYPE_hdhomerun_channelmap_record_t(Structure):
#    _fields_ = [('channelmap',            c_char_p),
#                ('range_list',            POINTER(TYPE_hdhomerun_channelmap_range_t)),
#                ('channelmap_scan_group', c_char_p),
#                ('countrycodes',          c_char_p)
#               ]

class TYPE_hdhomerun_tuner_status_t(Structure):
    _fields_ = [('channel',                 c_char * 32),
                ('lock_str',                c_char * 32),
                ('signal_present',          c_int, 32),
                ('lock_supported',          c_int, 32),
                ('lock_unsupported',        c_int, 32),
                ('signal_strength',         c_uint, 32),
                ('signal_to_noise_quality', c_uint, 32),
                ('symbol_error_quality',    c_uint, 32),
                ('raw_bits_per_second',     c_uint, 32),
                ('packets_per_second',      c_uint, 32)
               ]

    def __str__(self):
        return ("STATUS  CHANNEL= [%s]  LOCK_STR= [%s]  SIG_PRESENT= [%s]  "
                "LOCK_SUPP= [%s]  LOCK_UNSUPP= [%s]  SIG_STRENGTH= (%d)  "
                "SN_QUAL= (%d)  SYMERR_QUAL= (%d)  RAW_BPS= (%d)  PPS= (%d)" %
                (self.channel, self.lock_str, not not self.signal_present, 
                 not not self.lock_supported, not not self.lock_unsupported, 
                 self.signal_strength, self.signal_to_noise_quality, 
                 self.symbol_error_quality, self.raw_bits_per_second, 
                 self.packets_per_second))
               
class TYPE_hdhomerun_channelscan_program_t(Structure):
    _fields_ = [('program_str',    c_char * 64),
                ('program_number', c_uint, 16),
                ('virtual_major',  c_uint, 16),
                ('virtual_minor',  c_uint, 16),
                ('type',           c_uint, 16),
                ('name',           c_char * 32)
               ]

    def __str__(self):
        return ("PROGRAM  STR= [%s]  NUM= (%d)  VMAJ= (%d)  VMIN= (%d)  "
                "TYPE= (%d)  NAME= [%s]" % (self.program_str, 
                                            self.program_number, 
                                            self.virtual_major,
                                            self.virtual_minor,
                                            self.type,
                                            self.name))

class TYPE_hdhomerun_channelscan_result_t(Structure):
    _fields_ = [('channel_str',                  c_char * 64),
                ('channelmap',                   c_uint, 32),
                ('frequency',                    c_uint, 32),
                ('status',                       TYPE_hdhomerun_tuner_status_t),
                ('program_count',                c_int, 32),
                ('programs',                     TYPE_hdhomerun_channelscan_program_t * HDHOMERUN_CHANNELSCAN_MAX_PROGRAM_COUNT),
                ('transport_stream_id_detected', c_int, 32),
                ('transport_stream_id',          c_uint, 16)
               ]

    def __str__(self):
        return ("CHANNELSCAN-RES  STR= [%s]  MAP= (%d)  FREQ= (%d)  STATUS= "
                "[%s]  PROG_COUNT= (%d)  STREAM_DETECTED= [%s]  STREAM_ID= "
                "(%d)" % (self.channel_str, self.channelmap, self.frequency,
                          '<present>' if self.status else None, self.program_count, 
                          not not self.transport_stream_id_detected, 
                          self.transport_stream_id))

    def dump_programs(self):
        i = 0
        for i in xrange(self.program_count):
            print("%d: %s" % (i, self.programs[i]))

            i += 1

class TYPE_hdhomerun_channelscan_t(Structure):
    def __str__(self):
        return ("CHANNELSCAN  DEVICE= [%s]  SCANNED= (%d)  LIST= [%s]  "
                "NEXT= [%s]" % (self.hd.contents, self.scanned_channels, 
                                self.channel_list.contents, 
                                '<present>' if self.next_channel else None))

class TYPE_hdhomerun_device_t(Structure):
    @property
    def nice_multicast_ip(self):
        return ip_int_to_ascii(self.multicast_ip)

    @property
    def nice_device_id(self):
        return ("%X" % self.device_id)

    def __str__(self):
        return ("DEVICE  SCAN= [%s]  MC_IP= [%s]  MS_PORT= [%s]  "
                "ID= [%s]  TUNER= (%d)  LK= (%d)  NAME= [%s]  MODEL= [%s]" % 
                (self.scan.contents if self.scan else None, self.nice_multicast_ip, self.multicast_port, 
                 self.nice_device_id, self.tuner, self.lockkey, self.name,
                 self.model))

TYPE_hdhomerun_channelscan_t._fields_ = \
   [('hd',               POINTER(TYPE_hdhomerun_device_t)),
    ('scanned_channels', c_uint, 32),
    ('channel_list',     POINTER(TYPE_hdhomerun_channel_list_t)),
    ('next_channel',     POINTER(TYPE_hdhomerun_channel_entry_t))
   ]

class TYPE_hdhomerun_debug_message_t(Structure):
    pass

TYPE_hdhomerun_debug_message_t._fields_ = \
    [('next',   POINTER(TYPE_hdhomerun_debug_message_t)),
     ('prev',   POINTER(TYPE_hdhomerun_debug_message_t)),
     ('buffer', c_char * 2048)
    ]

#class TYPE_hdhomerun_debug_t(Structure);
#    _fields_ = [('thread', c_int, 32),
#                
#               ]

#struct hdhomerun_debug_t
#{
#	pthread_t thread;
#	volatile bool_t enabled;
#	volatile bool_t terminate;
#	char *prefix;
#
#	pthread_mutex_t print_lock;
#	pthread_mutex_t queue_lock;
#	pthread_mutex_t send_lock;
#
#	struct hdhomerun_debug_message_t *queue_head;
#	struct hdhomerun_debug_message_t *queue_tail;
#	uint32_t queue_depth;
#
#	uint64_t connect_delay;
#
#	char *file_name;
#	FILE *file_fp;
#	hdhomerun_sock_t sock;
#};

#class TYPE_hdhomerun_control_sock_t(Structure):
#    _fields_ = [
#                ('desired_device_id', c_uint, 32),
#                ('desired_device_ip', c_uint, 32),
#                ('actual_device_id', c_uint, 32),
#                ('actual_device_ip', c_uint, 32),
#                ('sock', c_int, 32),
#                ('dbg', POINTER(TYPE_hdhomerun_debug_t)),
#
#
#               ]
#
#struct hdhomerun_pkt_t {
#	uint8_t *pos;
#	uint8_t *start;
#	uint8_t *end;
#	uint8_t *limit;
#	uint8_t buffer[3074];
#};

#struct hdhomerun_control_sock_t {
#	uint32_t desired_device_id;
#	uint32_t desired_device_ip;
#	uint32_t actual_device_id;
#	uint32_t actual_device_ip;
#	uint sock;
#	struct hdhomerun_debug_t *dbg;
#	struct hdhomerun_pkt_t tx_pkt;
#	struct hdhomerun_pkt_t rx_pkt;
#};
#
#struct hdhomerun_video_sock_t {
#	pthread_mutex_t lock;
#	struct hdhomerun_debug_t *dbg;
#	hdhomerun_sock_t sock;
#
#	volatile size_t head;
#	volatile size_t tail;
#	uint8_t *buffer;
#	size_t buffer_size;
#	size_t advance;
#
#	pthread_t thread;
#	volatile bool_t terminate;
#
#	volatile uint32_t packet_count;
#	volatile uint32_t transport_error_count;
#	volatile uint32_t network_error_count;
#	volatile uint32_t sequence_error_count;
#	volatile uint32_t overflow_error_count;
#
#	volatile uint32_t rtp_sequence;
#	volatile uint8_t sequence[0x2000];
#};

TYPE_hdhomerun_device_t._fields_ = \
    [('cs',            POINTER(c_void_p)), # Not implemented, here.
    ('vs',             POINTER(c_void_p)),  # Not implemented, here.
    ('dbg',            POINTER(c_void_p)), # Not implemented, here.
    ('scan',           POINTER(TYPE_hdhomerun_channelscan_t)),
    ('multicast_ip',   c_uint, 32),
    ('multicast_port', c_uint, 16),
    ('device_id',      c_uint, 32),
    ('tuner',          c_uint, 32),
    ('lockkey',        c_uint, 32),
    ('name',           c_char * 32),
    ('model',          c_char * 32)
   ]

class TYPE_hdhomerun_tuner_vstatus_t(Structure):
    _fields_ = [
                ('vchannel',       c_char * 32),
                ('name',           c_char * 32),
                ('auth',           c_char * 32),
                ('cci',            c_char * 32),
                ('cgms',           c_char * 32),
                ('not_subscribed', c_int, 32),
                ('not_available',  c_int, 32),
                ('copy_protected', c_int, 32)
               ]

    @property
    def not_subscribed(self):
        return not not self.not_subscribed

    @property
    def not_available(self):
        return not not self.not_available

    @property
    def copy_protected(self):
        return not not self.copy_protected

    def __str__(self):
        return ("VSTATUS  VCHANNEL= [%s]  NAME= [%s]  AUTH= [%s]  CCI= [%s]  "
                "CGMS= [%s]  NOT_SUB= [%s]  NOT_AVAIL= [%s]  "
                "COPY_PROTECTED= [%s]" % (self.vchannel, self.name, self.auth,
                 self.cci, self.cgms, not not self.not_subscribed, 
                 not not self.not_available, not not self.copy_protected))




import os

HDHR_FILEPATH = os.environ['HDHOMERUN_PATH'] \
                    if 'HDHOMERUN_PATH' in os.environ \
                    else '/usr/lib/libhdhomerun.so'
                    
MAX_DEVICES = 16

HDHOMERUN_DEVICE_TYPE_TUNER             = 0x00000001
HDHOMERUN_DEVICE_ID_WILDCARD            = 0xFFFFFFFF
HDHOMERUN_CHANNELSCAN_MAX_PROGRAM_COUNT = 64

MAP_AU_BCAST = 'au-bcast'
MAP_AU_BCAST = 'au-cable'
MAP_EU_BCAST = 'eu-bcast'
MAP_EU_CABLE = 'eu-cable'
MAP_TW_BCAST = 'tw-bcast'
MAP_TW_CABLE = 'tw-cable'
MAP_KR_BCAST = 'kr-bcast'
MAP_KR_CABLE = 'kr-cable'
MAP_US_BCAST = 'us-bcast'
MAP_US_CABLE = 'us-cable'
MAP_US_HRC	 = 'us-hrc'
MAP_US_IRC   = 'us-irc'




import logging

from getpass import getuser

current_user = getuser()

if current_user == 'root':
    log_filepath = 'hdhomerun.log'

else:
    log_filepath = 'hdhomerun.log'

format = '%(asctime)s  %(levelname)s %(message)s'

logging.basicConfig(
        level       = logging.DEBUG,
        format      = format,
        filename    = log_filepath
    )

# Hook console logging.
log_console = logging.StreamHandler()
log_console.setLevel(logging.DEBUG)
log_console.setFormatter(logging.Formatter(format))

logging.getLogger('').addHandler(log_console)





from socket import inet_ntoa

def ip_int_to_ascii(ip_int):
    return inet_ntoa(hex(ip_int)[2:-1].zfill(8).decode('hex'))

def ip_ascii_to_int(ip):
    octets = [int(octet) for octet in ip.split('.')]

    if len(octets) != 4:
        raise Exception("IP [%s] does not have four octets." % (ip))

    encoded = ("%02x%02x%02x%02x" % (octets[0], octets[1], octets[2], octets[3]))
    return int(encoded, 16)




#!/usr/bin/python

from sys import exit
from pprint import pprint
from os import chdir

chdir('..')

from pyhdhomerun.adapter import HdhrUtility, HdhrDeviceQuery
from pyhdhomerun.constants import MAP_US_CABLE

def find_devices():
    devices = HdhrUtility.discover_find_devices_custom('192.168.5.11')
    
    for device in devices:
        print("Found: %s" % (device))

    return devices

def get_tuner_vstatus(device_adapter):
    
    (vstatus, raw_data) = device_adapter.get_tuner_vstatus()
    print(vstatus)

def set_tuner_vstatus(device_adapter, vchannel):
    
    device_adapter.set_tuner_vchannel(vchannel)

    (vstatus, raw_data) = device_adapter.get_tuner_vstatus()
    print(vstatus)

def set_stream(device_adapter, vchannel, target_uri):

    device_adapter.set_tuner_vchannel(vchannel)

    device_adapter.set_tuner_target(target_uri)

def get_supported(device_adapter):

    pprint(device_adapter.get_supported())

def scan(device_adapter):

    def found(result, scan_info):
        result.dump_programs()

    def progress_update(scan_info, progress):
        print("Progress: %.2f  Scanned= (%d)" % (progress, 
                                                 scan_info.scanned_channels))

    device_adapter.scan_channels(MAP_US_CABLE, found_cb=found, 
                                 incremental_cb=progress_update)

def get_count():

    return HdhrUtility.get_channels_in_range(MAP_US_CABLE)

devices = find_devices()

if not devices:
    print("Could not find any devices.")
    exit()

i = 0
for device in devices:
    print("%d: %s" % (i, device))
    i += 1
    
print

exit()

first_device_str = ("%s-%d" % (devices[0].nice_device_id, 1))

#create_device(first_device_str)

device_adapter = HdhrDeviceQuery(first_device_str)

get_tuner_vstatus(device_adapter)

#set_tuner_vstatus(device_adapter, 49)

set_stream(device_adapter, 49, 'rtp://192.168.5.102:7891')

#get_supported(device_adapter)

#print get_count()

#scan(device_adapter)






import logging

from ctypes import *
from sys import exit
from socket import inet_aton

from pyhdhomerun.externals import *
from pyhdhomerun.types import *
from pyhdhomerun.constants import MAX_DEVICES, HDHOMERUN_DEVICE_TYPE_TUNER, \
                                  HDHOMERUN_DEVICE_ID_WILDCARD
from pyhdhomerun.utility import ip_ascii_to_int

class HdhrUtility(object):
    """Calls that don't require a device entity."""

    @staticmethod
    def discover_find_devices_custom(ip=None):
        """Discover tuner devices."""

        try:
            ip_int = ip_ascii_to_int(ip) if ip else 0
        except:
            logging.exception("Could not convert IP [%s] to integer." % (ip))
            raise

        logging.info("Discovering devices.  MAX= (%d)  IP= [%s]" % 
                     (MAX_DEVICES, ip if ip_int else 0))

        devices = (TYPE_hdhomerun_discover_device_t * MAX_DEVICES)()

        try:
            num_found = CFUNC_hdhomerun_discover_find_devices_custom(
                            ip_int, 
                            HDHOMERUN_DEVICE_TYPE_TUNER, 
                            HDHOMERUN_DEVICE_ID_WILDCARD, 
                            devices, 
                            MAX_DEVICES
                        )
        except:
            logging.exception("Library call to discover devices failed.")
            raise

        if num_found == -1:
            message = ("Device discovery failed (%d)." % (num_found))

            logging.error(message)
            raise Exception(message)

        logging.info("(%d) devices found." % (num_found))

        return devices[0:num_found]

    @staticmethod
    def device_create_from_str(device_str):
        """Create a device-object to manipulate a specific device with."""

        logging.info("Creating device-entity for device [%s]." % (device_str))
    
        try:
            device = CFUNC_hdhomerun_device_create_from_str(device_str, 
                                                            None
                                                           )
        except:
            logging.exception("Library call to create device entity failed.")
            raise

        if not device:
            message = "Could not build device entity."
            
            logging.exception(message)
            raise Exception(message)

        return device.contents

    @staticmethod
    def get_channels_in_range(channel_map):
        """Determine the maximum number of channels available."""

        logging.info("Calculating channels count.")
    
        try:
            channel_list = CFUNC_hdhomerun_channel_list_create(channel_map)
        except:
            logging.exception("Could not create channel-list entity for range "
                              "check.")
            raise

        if not channel_list:
            message = "Could not build channel-list."
            
            logging.error(message)
            raise Exception(message)

        try:
            count = CFUNC_hdhomerun_channel_list_total_count(channel_list)
        except:
            logging.exception("Could not produce channel count.")
            raise
        else:
            return count
        finally:
            try:
                CFUNC_hdhomerun_channel_list_destroy(channel_list)
            except:
                logging.exception("Could not destroy channel-list entity.")
                raise

class HdhrDeviceQuery(object):
    """Device-specific queries."""

    device_str = None
    hd = None

    def __init__(self, device_str):
        self.device_str = device_str

        logging.info("Building device-query object for device [%s]." % 
                     (self.device_str))
    
        try:
            self.hd = HdhrUtility.device_create_from_str(self.device_str)
        except:
            logging.exception("Could not create device entity from string "
                              "[%s]." % (device_str))
            raise

    def __del__(self):
        try:
            CFUNC_hdhomerun_device_destroy(self.hd)
        except:
            logging.exception("Could not destroy device-entity object.")
            raise

    def get_tuner_vstatus(self):
        """Get the current state of the tuner using virtual channels (familiar 
        channel number).
        """

        logging.info("Doing device_get_tuner_vstatus call for device [%s]." % 
                     (self.device_str))

        raw_data = c_char_p()
        vstatus = TYPE_hdhomerun_tuner_vstatus_t()

        try:
            result = CFUNC_hdhomerun_device_get_tuner_vstatus(
                            self.hd, 
                            raw_data, 
                            vstatus
                        )
        except:
            logging.exception("Tuner vstatus failed.")
            raise

        if result != 1:
            message = ("Could not get tuner vstatus (%d)." % (result))
            
            logging.error(message)
            raise Exception(message)

        return (vstatus, raw_data.value)

    def set_tuner_vchannel(self, vchannel):
        """Set the current vchannel (familiar channel numbering)."""
        
        logging.info("Doing device_set_tuner_vstatus call for device [%s] with"
                     " vchannel [%s]." % (self.device_str, vchannel))
        
        try:
            result = CFUNC_hdhomerun_device_set_tuner_vchannel(self.hd, 
                                                               str(vchannel))
        except:
            logging.exception("Could not se vchannel.")
            raise

        if result != 1:
            message = "Failed to set vchannel."
            
            logging.error(message)
            raise Exception(message)
        
    def get_supported(self, prefix=None):
        """Get supported features as a dictionary of lists."""
    
        raw_str = c_char_p()

        logging.info("Doing device_get_supported call for device [%s]." % 
                     (self.device_str))

        try:
            result = CFUNC_hdhomerun_device_get_supported(
                            self.hd, 
                            None, 
                            raw_str
                        )
        except:
            logging.exception("Could not do get-supported request.")
            raise

        if result != 1:
            message = ("Could not get supported features (%d)." % (result))
            
            logging.error(message)
            raise Exception(message)

        raw_rows = raw_str.value.strip().split("\n")
        rows = { }
        for row in raw_rows:
            (key, value) = row.split(': ', 1)
            rows[key] = value.split(' ')

        return rows

    def scan_channels(self, channel_map, found_cb=None, incremental_cb=None):
        """Determine which channels can be locked. This is a long process and 
        the callbacks should be used in order to present progress to the user.
        """

        logging.info("Doing channel scan with map [%s]." % (channel_map))

        logging.debug("Determining range of channel scan.")

        try:
            num_channels = HdhrUtility.get_channels_in_range(channel_map)
        except:
            logging.exception("Could not calculate the maximum number of "
                              "channels to be scanned.")
            raise

        logging.debug("Building channel-scan object.")

        try:
            scan = CFUNC_channelscan_create(self.hd, channel_map)
        except:
            logging.exception("Could not initialize channel-scan object.")
            raise

        if not scan:
            message = "Could not build channel-scan object."
            
            logging.error(message)
            raise Exception(message)

        logging.debug("Doing actual scan.")

        try:
            found = self.__do_scan(scan, num_channels, found_cb, 
                                   incremental_cb)
        except:
            logging.exception("Could not do actual channel scan.")
            raise
        else:
            logging.info("Found programs on (%d) channels." % (len(found)))
            return found
        finally:
            try:
                CFUNC_channelscan_destroy(scan)
            except:
                logging.exception("Could not destroy channel-scan entity.")
                raise

    def __do_scan(self, scan, num_channels, found_cb=None, 
                  incremental_cb=None):
        """Do the actual scan (looping over channel numbers)."""
    
        i = 0
        num_channels = float(num_channels)
        found = []
        while 1:
            result = TYPE_hdhomerun_channelscan_result_t()

            if CFUNC_channelscan_advance(scan.contents, result) != 1:
                break

            elif CFUNC_channelscan_detect(scan.contents, result) == 1:
                if result.program_count > 0:
                    found.append(result)
                
                    if found_cb:
                        logging.info("Invoking 'found' callback.")

                        try:
                            found_cb(result, scan.contents)
                        except:
                            logging.exception("The 'found' callback threw an "
                                              "exception.")
                            raise
            i += 1

            progress = (float(i) / num_channels * 100.0)

            if incremental_cb:
                logging.info("Invoking 'incremental' callback.")
            
                try:
                    incremental_cb(scan.contents, progress)
                except:
                    logging.exception("The 'incremental' callback threw an "
                                      "exception.")
                    raise

            logging.info("Channel scan progress is (%.2f)%%." % (progress))

        return found

    def set_tuner_target(self, target_uri):
        """Start sending video to the given URI."""
# If doesn't work: int hdhomerun_device_stream_start(struct hdhomerun_device_t *hd)

        logging.info("Setting target to [%s]." % (target_uri))

        try:
            result = CFUNC_hdhomerun_device_set_tuner_target(self.hd, 
                                                             target_uri)
        except:
            logging.exception("There was an exception while setting the tuner "
                              "target.")
            raise

        if result != 1:
            message = ("Could not set tuner target to [%s]." % (target))
            
            logging.error(message)
            raise Exception(message)




from ctypes import *

from pyhdhomerun.hdhr import get_hdhr
from pyhdhomerun.types import *
from pyhdhomerun.constants import MAX_DEVICES

try:
    hdhr = get_hdhr()
except:
    logging.exception("Could not acquire HDHR object for bindings.")
    raise

CFUNC_hdhomerun_discover_find_devices_custom = \
    hdhr.hdhomerun_discover_find_devices_custom
CFUNC_hdhomerun_discover_find_devices_custom.argtypes = \
    [c_uint, 
     c_uint, 
     c_uint, 
     TYPE_hdhomerun_discover_device_t * MAX_DEVICES, 
     c_int
    ]

CFUNC_hdhomerun_device_create_from_str = \
    hdhr.hdhomerun_device_create_from_str
CFUNC_hdhomerun_device_create_from_str.argtypes = \
    [c_char_p, 
     c_void_p
    ]
CFUNC_hdhomerun_device_create_from_str.restype = \
    POINTER(TYPE_hdhomerun_device_t)

CFUNC_hdhomerun_device_destroy = \
    hdhr.hdhomerun_device_destroy
CFUNC_hdhomerun_device_destroy.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t)
    ]
CFUNC_hdhomerun_device_destroy.restype = \
    None

CFUNC_hdhomerun_device_get_tuner_vstatus = \
    hdhr.hdhomerun_device_get_tuner_vstatus
CFUNC_hdhomerun_device_get_tuner_vstatus.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t),
     POINTER(c_char_p),
     POINTER(TYPE_hdhomerun_tuner_vstatus_t)
    ]

CFUNC_hdhomerun_device_get_supported = \
    hdhr.hdhomerun_device_get_supported
CFUNC_hdhomerun_device_get_supported.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t), 
     c_void_p,
     POINTER(c_char_p)
    ]

#CFUNC_hdhomerun_channel_list_create = \
#    hdhr.hdhomerun_channel_list_create
#CFUNC_hdhomerun_channel_list_create.argtypes = 
#    [ c_char_p ]
#CFUNC_hdhomerun_channel_list_create.restype = 
#    POINTER(TYPE_hdhomerun_channel_list_t)
#
#CFUNC_hdhomerun_channel_list_first = \
#    hdhr.hdhomerun_channel_list_first
#CFUNC_hdhomerun_channel_list_first.argtypes = \
#    [ POINTER(TYPE_hdhomerun_channel_list_t) ]
#CFUNC_hdhomerun_channel_list_first.restype = \
#    POINTER(TYPE_hdhomerun_channel_entry_t)

CFUNC_channelscan_create = \
    hdhr.channelscan_create
CFUNC_channelscan_create.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t), 
     c_char_p
    ]
CFUNC_channelscan_create.restype = \
     POINTER(TYPE_hdhomerun_channelscan_t)

CFUNC_channelscan_destroy = \
    hdhr.channelscan_destroy
CFUNC_channelscan_destroy.argtypes = \
    [POINTER(TYPE_hdhomerun_channelscan_t)
    ]
CFUNC_channelscan_destroy.restype = \
    None

CFUNC_channelscan_advance = \
    hdhr.channelscan_advance
CFUNC_channelscan_advance.argtypes = \
    [POINTER(TYPE_hdhomerun_channelscan_t),
     POINTER(TYPE_hdhomerun_channelscan_result_t)
    ]

CFUNC_channelscan_detect = \
    hdhr.channelscan_detect
CFUNC_channelscan_detect.argtypes = \
    [POINTER(TYPE_hdhomerun_channelscan_t),
     POINTER(TYPE_hdhomerun_channelscan_result_t)
    ]

CFUNC_hdhomerun_channel_list_create = \
    hdhr.hdhomerun_channel_list_create
CFUNC_hdhomerun_channel_list_create.argtypes = \
    [c_char_p
    ]
CFUNC_hdhomerun_channel_list_create.restype = \
    POINTER(TYPE_hdhomerun_channel_list_t)

CFUNC_hdhomerun_channel_list_destroy = \
    hdhr.hdhomerun_channel_list_destroy
CFUNC_hdhomerun_channel_list_destroy.argtypes = \
    [
    POINTER(TYPE_hdhomerun_channel_list_t)
    ]
CFUNC_hdhomerun_channel_list_destroy.restype = \
    None

CFUNC_hdhomerun_channel_list_total_count = \
    hdhr.hdhomerun_channel_list_total_count
CFUNC_hdhomerun_channel_list_total_count.argtypes = \
    [
    POINTER(TYPE_hdhomerun_channel_list_t)
    ]
CFUNC_hdhomerun_channel_list_total_count.restype = \
    (c_uint32)

CFUNC_hdhomerun_device_set_tuner_vchannel = \
    hdhr.hdhomerun_device_set_tuner_vchannel
CFUNC_hdhomerun_device_set_tuner_vchannel.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t),
     c_char_p
    ]

CFUNC_hdhomerun_device_set_tuner_target = \
    hdhr.hdhomerun_device_set_tuner_target
CFUNC_hdhomerun_device_set_tuner_target.argtypes = \
    [POINTER(TYPE_hdhomerun_device_t),
     c_char_p
    ]





import logging

from ctypes import *

from pyhdhomerun import constants

def get_hdhr():
    """Get an instance of the library."""

    try:
        return get_hdhr.instance
    except:
        pass

    try:
        get_hdhr.instance = cdll.LoadLibrary(constants.HDHR_FILEPATH)
    except:
        logging.exception("Could not load HDHR library from [%s]." % 
                          (constants.HDHR_FILEPATH))
        raise

    return get_hdhr.instance




from setuptools import setup, find_packages
import sys, os

version = '2.2.1'

setup(name='pyhdhomerun',
      version=version,
      description="HDHomeRun interface library.",
      long_description="""\
HDHomeRun interface library. Supports device discovery, channel-scanning, streaming, status inquiries, channel changes, etc..""",
      classifiers=['Development Status :: 4 - Beta',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: English',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   'Topic :: Multimedia :: Video :: Capture'
                  ],
      keywords='tv television tuner tvtuner hdhomerun',
      author='Dustin Oprea',
      author_email='myselfasunder@gmail.com',
      url='https://github.com/dsoprea/PyHdHomeRun',
      license='New BSD',
      packages=['pyhdhomerun'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools'
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
